#!/usr/bin/perl -w

########################################################################
#
# program: blogcal.cgi
# system : Blog Calendar Mod for WebAPP NE (Network Edition)
# author : Copyright (C) 2006 Steven L. Reid
# purpose: Display a calendar of articles (sidebar).
# history:
# 12/02/2006, Steven Reid
#	- script created
#
########################################################################

# setup environment
$|++;
#use strict; # turned off due to WebAPP not supporting it

# Read configuration file
BEGIN {
  use CGI::Carp qw(fatalsToBrowser);
  # include initial configuration
  require "$scriptdir/mods/blogcal/data/blogcal-paths.cfg";
  require "$scriptdir/mods/blogcal/data/blogcal.cfg";
  # include language file (if exists)
  if ( -e "$bc_langDir/$userlang") {
    require "$bc_langDir/$userlang";
  } else {
    require "$bc_langDir/$bc_defaultLang";
  }
}

# load libraries
use File::Basename;

# define globals
my @MONTH = split(/\s+/, $bc_msg{'008'});
my @DAY = split(/\s+/, $bc_msg{'009'});
my $URL = "$bc_scriptURL/index.cgi?";

# define variables
my $bc_month; my $bc_day; $bc_year;
my $bc_weekday       =  0;
my $bc_monthlen      = 31;
my @bc_blog_dates    = ();
my $bc_next_year; my $bc_next_month;
my $bc_prev_year; my $bc_prev_month;
my $selfurl = $ENV{'REQUEST_URI'} || $URL;
$selfurl .= ($selfurl !~ /\?/ ? '?' : '&amp;'); 

########################################################################
# Display Blog Calendar
########################################################################
sub display_blog_calendar {
  bc_load_parms();		# load any passed parameters
  bc_do_math();			# perform date math
  bc_get_blogs();		# get blogs for given month
#  boxheader("$bc_title" );
  bc_print_cal();		# print the calendar
#  boxfooter();
}
########################################################################

########################################################################
# Load params - get the form variables if need be
########################################################################
sub bc_load_parms {
  # load parameters
  my $m = $info{'bc_month'} or return;
  my $y = $info{'bc_year'} or return;
  $bc_month = $m;
  $bc_year = $y;
  $bc_day = 0;
}
########################################################################

########################################################################
# Perform date math (leap year, month length and week day of the first)
########################################################################
sub bc_do_math {
  # grab today if month not passed
  getTimezones(); # convert to current time
  my ($m, $d, $y) = getNewtime(split(/\s-\s/, $date));
  $y+=100 if $y < 100; $y+=1900;
  if ($bc_month) {
    # set day if month, year are now
    $bc_day = $d if $bc_month == $m && $bc_year == $y;
  } else {
    $bc_month = $m; $bc_day = $d; $bc_year = $y;
  }

  # set prev/next information
  $next_year     = $bc_month == 12 ? $bc_year+1 : $bc_year;
  $prev_year     = $bc_month ==  1 ? $bc_year-1 : $bc_year;
  $next_month    = $bc_month == 12 ?  1 : $bc_month+1;
  $prev_month    = $bc_month ==  1 ? 12 : $bc_month-1;

  $bc_month--;	# make it unix format month (subtract 1)

  # determine leap year
  my $leap = 0;
  $leap = 1 unless $bc_year % 4;
  $leap = 0 unless $bc_year % 100;
  $leap = 1 unless $bc_year % 400;

  # determine length of month
  $bc_monthlen = 28 + $leap if $bc_month == 1;	# February
  $bc_monthlen-- if $bc_month =~ /3|5|8|10/;	# months with 30 days

  # week day of the first of the month using Zeller's Congruence
  $m = $bc_month + 1; $d = 1; $y = $bc_year;
  if ($m < 3) { $m+=12; $y--; }
  $bc_weekday = ( $d + $m*2 + int(($m+1)*0.6) + 1 + $y +
               int($y/4) - int($y/100) + int($y/400)  ) % 7;
}
########################################################################

########################################################################
# Get Blogs - gets a list of blogs and sorts them according to date/time
########################################################################
sub bc_get_blogs {
  # walk blog tree, matching any criteria and placing into
  # @blog
  my $m = $bc_month + 1;  $m = "0$m" if $m < 10;
  my $y = $bc_year - 1900; $y -= 100 if $y > 100; $y = "0$y" if $y < 10;

  # get list of articles
  opendir (DIR, "$topicsdir/articles");
  my @files = readdir(DIR);
  closedir (DIR);
  @files = grep (/\.txt/, @files);
  
  # walk list of files, looking for matches
  for my $file (@files) {
    open (FILE, "<$topicsdir/articles/$file");
    hold(FILE);
    my $head = <FILE>;
    release(FILE);
    close(FILE);
    my ($adate) = (split ( /\|/, $head))[4];
    # format: 02/25/06 - 22:53:30
    my ($month, $day, $year)
      = getNewtime(split(/\s-\s/, $adate));
    next unless $month == $m && $year == $y;
    $bc_blog_dates{$day}->{'day'} = "$year$month$day&amp;bc_month=$month&amp;bc_year=$bc_year";
    $bc_blog_dates{$day}->{'cnt'}++;
  }
}
########################################################################

########################################################################
# Print a nice HTML calendar
########################################################################
sub bc_print_cal {
  my $d;			# day of month
  my $c = $bc_weekday-1;	# calendar day of month (used for ends)
  my ($h, $h1, $h2);		# holiday string
  my %WEEKEND = (7=>1,14=>1,21=>1,28=>1,35=>1,42=>1);
  my $next = "${selfurl}bc_year=$next_year&amp;bc_month=$next_month";
  my $prev = "${selfurl}bc_year=$prev_year&amp;bc_month=$prev_month";

  # display header
  if (!defined &boxtop) { # old themes compatibility
    print qq~<tr><td>~;
  }

  # print top of calendar
  my $title = "$MONTH[$bc_month]  $bc_year";
  $title =~ s/\s/&nbsp;/g;
  print <<END_HTML;
    <div style="$bc_style_blogcal" align="center">
      <p style="$bc_style_title">$title</p>
      <table summary="$title" style="$bc_style_month" align="center">
	<tr>
END_HTML
  for my $day (@DAY) {
    print qq(<th style="$bc_style_week">$day</th>);
  }
  print qq(</tr>\n);

  # print days of month
  print qq(<tr>);
  # print any unused days at start of month
  print qq(<td style="$bc_style_day" bgcolor="$bc_noday_color">&nbsp;</td>) x $bc_weekday;
  # print each day in month
  for $d (1..$bc_monthlen) {
    $c++;
    $zd = ($d < 10 ? "0$d" : $d);
    # is this the end of the week?
    print qq(</tr>\n<tr>) if $WEEKEND{$c};
    # is this a blog, or a regular day?
    if ($bc_blog_dates{$zd}) {
      my $entry = $bc_blog_dates{$zd}->{'day'};
      my $cnt = $bc_blog_dates{$zd}->{'cnt'};
      my $posts = "$cnt "
        . ($cnt == 1 ? $bc_msg{'010'} : $bc_msg{'002'});
      my %bc_blog_date = %{$bc_blog_dates{$zd}};
      print qq(<td style="$bc_style_daypost" bgcolor="),
        ($d == $bc_day ? "$bc_today_color" :
        $WEEKEND{$c} || $WEEKEND{$c+1} ? "$bc_weekend_color"
	: "$bc_day_color");
      print qq("><a href="${URL}date=$entry"),
        qq( title="$posts">$d</a></td>);
    } else {
      print qq(<td style="$bc_style_day" bgcolor="), ($d == $bc_day ? "$bc_today_color" :
        $WEEKEND{$c} || $WEEKEND{$c+1} ? "$bc_weekend_color"
	: "$bc_day_color");
      print qq(">$d</td>);
    }
  }
  while (not $WEEKEND{++$c}) {
    print qq(<td bgcolor="$bc_noday_color">&nbsp;</td>);
  }
  print qq(</tr><tr>),
    qq(<td bgcolor="$bc_noday_color">&nbsp;</td>) x 7 if $c <= 35;
  print <<END_HTML;
        </tr>
	<tr>
	  <td colspan="7" style="$bc_style_day">
	    <span style="float: left;"><a href="$prev"
	      style="$bc_style_navigation"
	      title="$bc_msg{'013'}">&lt;&lt;&nbsp;$bc_msg{'011'}</span>
	    <span style="float: right;"><a href="$next"
	      style="$bc_style_navigation"
	      title="$bc_msg{'014'}">$bc_msg{'012'}&nbsp;&gt;&gt;</span>
	  </td>
	</tr>
      </table>
    </div>
END_HTML
  if (!defined &boxbottom) { # old themes compatibility
    print qq~</td></tr>~;
  }
}
########################################################################

1;
